/*
 * FoncNum1D.cpp
 *
 *  Created on: 13 nov. 2009
 *      Author: Anya
 */

#include "FoncNum1D.h"


/*
 * Premier constructeur de la classe FoncNum1D
 * Il permet de crer une fonction numrique  partir d'un tableau de valeurs numriques
 * et d'un pointeur sur une grille
 * le nombre de points de la grille doit tre gal au nombre de valeurs dans le tableau
 * Dans ce cas on considre que la i-me valeur v(i) correspond  f(x(i)),
 * la valeur de notre fonction au point
 * x(i)
 */
FoncNum1D::FoncNum1D(Grille1D * gr, vector<double> v)
{
	if(gr->getNbPoints()!=(int)v.size())//pointeur au point on met dirig vers ->
	{//vrification des dimensions
		cout<<"donnes incopatibles!\n";
		exit(0);
	}
	grille=gr;
	vals.resize(v.size());// rservation de la place en mmoire pour copier les valeurs  de v
	vals=v;               // oprateur de copie pour la classe vector
	// il permet de copier un vecteur dans un autre  condition qu'ils soient de la mme taille
}

/*
 * Une fonction numrique peut aussi tre construite
 * par chantillonnage d'une fonction relle, analytique, connue
 * pour cela nous dfinissons ici une deuxime constructeur (surcharge)
 * qui reoit en paramtre une rfrence  une grille
 * et un pointeur  sur une fonction que l'on souhaite
 * ainsi discrtiser
 */
FoncNum1D::FoncNum1D(Grille1D * gr, double (*ptrF)(double))
{
	grille=gr;
	/*
	 * Pour rcuprer le nombre de points de la grille on passe par une mthode d'accs
	 * en cas d'appels multiples il peut tre prfrable de ranger la valeur dans une variable
	 * comme c'est fait ici
	 */
	int nbPoints=grille->getNbPoints();
	vals.resize(nbPoints); // rservation  de mmoire ncessaire pour stocker le valeurs de la fonction
	/*
	 * On rcupre ensuite les points de la grille dans une variable  de type vector<double>
	 */
	vector<double> lesX(nbPoints);
	lesX=gr->getPoints();

	for(int i=0;i<nbPoints;i++)
	{
		/*
		 * ici on appelle la fonction par sa rfrence :
		 *  (*nomDuPointeur)(liste des arguments)
		 *  Pour chaque point de la grille on calcule la faleur de la fonction
		 *
		 */
		vals[i]=(*ptrF)(lesX[i]);//
	}
}
/*
 * une mthode d'accs  l'attribut vals
 */
vector<double> FoncNum1D::getVals()
{
	return vals;
}

/*
 * Mthode qui permet d'exporter les valeurs de la fonction numrique pour une visualisation graphique
 *  l'aide de Scilab
 * Prend  en entre une chane de caractre : le nom du fichier de sortie
 *
 * En C++, contrairement  C, il eiste une classe standard string
 * pour manipuler les chanes de caractres
 * voir la documentation sur le site de rfrence
 */
void FoncNum1D::exportScilab(string nomFichier )
{
	/*
	 * Dclaration d'un objet de sortie ofstream
	 * correspond  l'ouverture d'un fichier pour l'criture
	 */
	ofstream fichierScilab(nomFichier.c_str());

	if(fichierScilab)  // si l'ouverture a russi
	{
		/*
		 * Si l'ouverture se passe bien
		 * on rcupre les points de la grille dans une vecteur
		 */

		int nbPoints=grille->getNbPoints();
		vector<double> pointsGrille(nbPoints);
		pointsGrille=(grille->getPoints());

		/*
		 * on utilise ici l'oprateur d'insertion formte pour un flux d'entre/sortie
		 * cet oprateur s'utilise de la mme manire avec n'iporte quel objet de type flux
		 * d'entre/sortie (fichier, console etc)
		 * la syntaxe gnrale est
		 * nomFlux << nomvariable;
		 * le format d'insertion sera automatiquement dduit du type de la donne inserre
		 *
		 * ici le flux est un fichier de sortie, son nom est fichierScilab
		 *
		 * On imprime par exemple une chane de caractres " x=["
		 */
		fichierScilab << "  x=[";
		for(int i=0;i<nbPoints;i++)
		{
			/*
			 * On crit ensuite les valeurs des points de la grille (des doubles)
			 */
			fichierScilab<<" "<<pointsGrille[i];
		}
		/*
		 * On termine la commande scilab qui consiste  dclarer un tableau x
		 * en numrant ses lments
		 */
		fichierScilab<<" ];\n";
/*
 * On procde de manire analogue pour crire les valeurs de la fonction dans un deuxime tableau, y
 */
		fichierScilab << "  y=[";
		for(int i=0;i<nbPoints;i++)
		{
			fichierScilab<<" "<<vals[i];
		}
		/*
		 * On ajoute une isntruction de construction d'un  graphique
		 */
		fichierScilab<<" ];\n  plot2d(x,y);\n";


		fichierScilab.close();  // je referme le fichier
	}
	else  // sinon
		cerr << "Erreur  l'ouverture !" << endl;
	cout<<"fichier fini\n";

}
FoncNum1D::~FoncNum1D() {
	// TODO Auto-generated destructor stub
}
